/******************************************************************************
						jsFileService Object
*******************************************************************************/	

function jsFileService(input_xml) {
	this.request      = input_xml;
	this.response     = xmlCreateDocument("Response");
	this.responseList = xmlAppendChild(this.response, this.response.documentElement, "CommandList", null);
	
	this.processRequest = jsFileService_processRequest;
	this.processRequest = jsFileService_processRequest;
	this.processCommand = jsFileService_processCommand;
	this.processDelete  = jsFileService_processDelete;
	this.processMkDir   = jsFileService_processMkDir;
	this.processCopy    = jsFileService_processCopy;  
	this.processMove    = jsFileService_processMove;  
	this.processList    = jsFileService_processList;
}

function jsFileService_processRequest() {
	with (this) {
		var cmdList = request.documentElement.firstChild.childNodes;
		for(var i=0; i<cmdList.length; i++) {
			processCommand(cmdList.item(i), xmlAppendChild(response, responseList, "Command", null));
		}
	}
}
	
function jsFileService_processRequest() {
	with (this) {
		var cmdList = request.documentElement.firstChild.childNodes;
		for(var i=0; i<cmdList.length; i++) {
			processCommand(cmdList.item(i), xmlAppendChild(response, responseList, "Command", null));
		}
	}
}
	
function jsFileService_processCommand(xmlCmd, outItem) {
	with(this) {
		var params = xmlCmd.childNodes;
		var name   = params.item(0).text;
		var src    = params.item(1);

		xmlAppendChild(response, outItem, "Name",   name);
		xmlAppendChild(response, outItem, "Status", "Ok");
		var outRoot = xmlAppendChild(response, outItem, "Output", null);
		
		switch(name) {
			case "Delete" : 
				processDelete(src, outRoot);
				break;
			case "Copy" :
				processCopy(src, params.item(2), params.item(3), outRoot)
				break;
			case "Move" :
				processMove(src, params.item(2), params.item(3), outRoot)
				break;
			case "List" :
				processList(src, outRoot);
				break;
			case "MkDir" :
				processMkDir(src, outRoot);
				break;
		} // switch
	} // with
} 

function jsFileService_processDelete(src, outItem) {
	with(this){
		var nodes = src.childNodes;
		for(var i=0; i<nodes.length; i++) {
			var path = nodes.item(i).firstChild.text;
			xmlAppendChild(response, outItem, "Path", path);
			switch (nodes.item(i).nodeName) {
				case "Folder" :
					deleteFolder(path);
					break;
				case "File"   :
					deleteFile(path);
					break;
			} // switch
			var err = getLastError();
			if (err != null) {
				xmlAppendChild(response, outItem, "Exception", err);
			}
		} // for
	} // with
}

function jsFileService_processMkDir(src, outItem) {
	with(this) {
		var nodes = src.childNodes;
		var path  = nodes.item(0).firstChild.text;
		xmlAppendChild(response, outItem, "Path", path);
		if (createFolder(nodes.item(0).firstChild.text) == null) {
			xmlAppendChild(response, outItem, "Exception", getLastError());
		}
	}
}
	
function jsFileService_processCopy(src, dest, filter, outItem) {
	with(this) {
		var nodes = src.childNodes;
		var copy_to  = dest.firstChild.text;
		xmlAppendChild(response, outItem, "Destination", copy_to);
		for(var i=0; i<nodes.length; i++) {
			var path     = nodes.item(i).firstChild.text;
			xmlAppendChild(response, outItem, "Item", path);
			switch (nodes.item(i).nodeName) {
				case "Folder" :
					copyFolder(path, copy_to);
					break;
				case "File"   :
					copyFile(path, copy_to);
					break;
			} // switch
			var err = getLastError();
			if (err != null) {
				xmlAppendChild(response, outItem, "Exception", err);
			}
		} // for
	}
}

function jsFileService_processMove(src, dest, filter, outItem) {
	with(this) {
		var nodes = src.childNodes;
		var copy_to  = dest.firstChild.text;
		xmlAppendChild(response, outItem, "Destination", copy_to);
		for(var i=0; i<nodes.length; i++) {
			var path     = nodes.item(i).firstChild.text;
			xmlAppendChild(response, outItem, "Item", path);
			switch (nodes.item(i).nodeName) {
				case "Folder" :
					moveFolder(path, copy_to);
					break;
				case "File"   :
					moveFile(path, copy_to);
					break;
			} // switch
			var err = getLastError();
			if (err != null) {
				xmlAppendChild(response, outItem, "Exception", err);
			}
		} // for
	}
}

function jsFileService_processList(src, root) {
	with(this) {
		var nodes = src.childNodes;
		var path  = nodes.item(0).firstChild.text;
		getFileListXML(nodes.item(0).firstChild.text, response, root) 
	}
}

/******************************************************************************
						jsFileServiceCommand Object
*******************************************************************************/	

function jsFileServiceCommand(name) {
	this.name        = name;
	this.folders     = new Array();
	this.files       = new Array();
	this.filter      = null;
	this.destination = null;
}		

/******************************************************************************
						jsFileServiceRequest Object
*******************************************************************************/	
function jsFileServiceRequest() {
	this.request     = xmlCreateDocument("Request");
	this.response    = null;
	this.requestList = xmlAppendChild(this.request, this.request.documentElement, "CommandList", null);
	
	this.addCommand = jsFileServiceRequest_addCommand;
	this.addList    = jsFileServiceRequest_addList;
	this.addMkDir   = jsFileServiceRequest_addMkDir;
	this.send       = jsFileServiceRequest_send;
}

function jsFileServiceRequest_addCommand(cmd) {
	with (this) {
		var xCmd    = xmlAppendChild(request, requestList, "Command", null);
		var xName   = xmlAppendChild(request, xCmd,        "Name",    cmd.name);
		var xSrc    = xmlAppendChild(request, xCmd,        "Source",  null);
		
		for(var i=0; i<cmd.folders.length; i++) {
			xmlAppendChild(request, xSrc, "Folder", cmd.folders[i]);
		}
		for(var i=0; i<cmd.files.length; i++) {
			xmlAppendChild(request, xSrc, "File", cmd.files[i]);
		}
		if(cmd.destination != null) {
			var xDest   = xmlAppendChild(request, xCmd, "Destination", cmd.destination);
		}
		if(cmd.filter != null) {
			var xFilter = xmlAppendChild(request, xCmd, "Filter",      cmd.filter);
		}
	}
}
	
function jsFileServiceRequest_addList(path) {
	with(this) {
		var cmd = new jsFileServiceCommand("List");
		cmd.folders[0] = path;
		addCommand(cmd);
	}
}

function jsFileServiceRequest_addMkDir(path) {
	with(this) {
		var cmd = new jsFileServiceCommand("MkDir");
		cmd.folders[0] = path;
		addCommand(cmd);
	}
}

function jsFileServiceRequest_send(service) {
	var xmlhttp = null;
	with(this) {
		if(service == "local") {
			var fService = new jsFileService(request);
			fService.processRequest();
			response = fService.response;	
		} else {
	    	var xmlhttp = xmlHTTP();
	    	xmlhttp.Open("POST", service, false);
	    	xmlhttp.Send(request.xml);
			response = xmlhttp.responseXML;
		}
	}
}

